﻿// (C) Copyright 2011 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software
// in object code form for any purpose and without fee is hereby
// granted, provided that the above copyright notice appears in
// all copies and that both that copyright notice and the limited
// warranty and restricted rights notice below appear in all
// supporting documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,
// INC. DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL
// BE UNINTERRUPTED OR ERROR FREE. 
//
// Use, duplication, or disclosure by the U.S. Government is
// subject to restrictions set forth in FAR 52.227-19 (Commercial
// Computer Software - Restricted Rights) and DFAR 252.227-7013(c)
// (1)(ii)(Rights in Technical Data and Computer Software), as
// applicable.
//


using System;
using System.Collections.Generic;
using System.Linq;

using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Architecture;
using Autodesk.Revit.UI;
using Autodesk.Revit.ApplicationServices;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.UI.Selection;


/* Lab5 can duplicate a specified group to the position user picked.
 * It shows the usage of following basic APIs 
 * 
 * Filter to select specified elements
 * Try-Catch usage.
 *  
 * Before run this command, a group should  already be in the active
 * document.
 */

[TransactionAttribute(TransactionMode.Manual)]
[RegenerationAttribute(RegenerationOption.Manual)]
public class Lab5PickFilter : IExternalCommand
{
  public Result Execute(
    ExternalCommandData commandData,
    ref string message,
    ElementSet elements)
  {
    UIApplication uiApp = commandData.Application;    
    Document doc = uiApp.ActiveUIDocument.Document;

    try
    {
      //Define a Reference object to accept the pick result.
      Reference pickedRef = null;      

      //Pick a group using a filter.
      //So only group can be selected.
      Selection sel = uiApp.ActiveUIDocument.Selection;
      GroupPickFilter selFilter = new GroupPickFilter();
      pickedRef = sel.PickObject(ObjectType.Element, selFilter, 
        "Please select a group");
      Element elem = doc.GetElement(pickedRef);
      Group group = elem as Group;

      //Pick a point
      XYZ point = sel.PickPoint("Please pick a point");

      //Place the group
      Transaction trans = new Transaction(doc);
      trans.Start("Lab");
      doc.Create.PlaceGroup(point, group.GroupType);
      trans.Commit();
    }

    //If user right click or press ESC button, handle the exception
    catch (Autodesk.Revit.Exceptions.OperationCanceledException)
    {      
      return Result.Cancelled;
    }
    //Catch other error.
    catch (Exception ex)
    {
      message = ex.Message;
      return Result.Failed;
    }

    return Result.Succeeded;
  }
}


/// <summary>
/// Filter to constrain picking to model groups. Only model groups 
/// are highlighted and can be selected when cursor is hovering.
/// </summary>
public class GroupPickFilter : ISelectionFilter
{
  public bool AllowElement(Element e)
  {
    return (e.Category.Id.IntegerValue.Equals(
      (int)BuiltInCategory.OST_IOSModelGroups));
  }
  public bool AllowReference(Reference r, XYZ p)
  {
    return false;
  }
}
